<?php

declare(strict_types=1);

namespace mirzaev\ebala\models;

// Project files
use mirzaev\ebala\models\traits\instance,
	mirzaev\ebala\models\traits\status,
	mirzaev\ebala\models\account,
	mirzaev\ebala\models\worker,
	mirzaev\ebala\models\market;

// Фреймворк ArangoDB
use mirzaev\arangodb\collection;

// Библиотека для ArangoDB
use ArangoDBClient\Document as _document;

// Встроенные библиотеки
use exception;

/**
 * Модель реестра
 *
 * Генерирует HTML-документы из данных аккаунтов (accounts), магазинов (markets) и сотрудников (workers)
 *
 * @package mirzaev\ebala\models
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class registry extends core
{
	use instance, status;

	/**
	 * Generate workers list
	 *
	 * @param ?string $before Injection of AQL-code before search
	 * @param int $amount Amount of workers
	 * @param int $page Offset by amount
	 * @param array $errors Errors registry
	 *
	 * @return array Instances from ArangoDB
	 */
	public static function workers(
		?string $before = '',
		int $amount = 100,
		int $page = 1,
		string $sort = 'worker.created DESC',
		array &$errors = []
	): array {
		try {
			if (collection::init(static::$arangodb->session, account::COLLECTION) && collection::init(static::$arangodb->session, worker::COLLECTION)) {
				// Инициализированы коллекции

				// Search the session data in ArangoDB
				$workers = collection::search(static::$arangodb->session, sprintf(
					<<<AQL
						FOR account IN %s
							FILTER account.status != 'deleted'
							%s
							LET worker = (FOR worker IN OUTBOUND account._id account_edge_worker FILTER worker.status != 'deleted' LIMIT 1 SORT worker.created DESC RETURN worker)[0]
							FILTER worker != null
							SORT %s
							LIMIT %d, %d
							RETURN {account, worker}
					AQL,
					account::COLLECTION,
					$before,
					$sort,
					--$page <= 0 ? 0 : $amount * $page,
					$amount
				));

				// Exit (success)
				return empty($workers) ? [] : (is_array($workers) ? $workers : [$workers]);
			} else throw new exception('Не удалось инициализировать коллекции');
		} catch (exception $e) {
			// Write to the errors registry
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Exit (fail)
		return [];
	}

	/**
	 * Generate markets list
	 *
	 * @param ?string $before Injection of AQL-code before search
	 * @param int $amount Amount of markets
	 * @param int $page Offset by amount
	 * @param array $errors Errors registry
	 *
	 * @return array Instances from ArangoDB
	 */
	public static function markets(
		?string $before = '',
		int $amount = 100,
		int $page = 1,
		string $sort = 'market.created DESC',
		array &$errors = []
	): array {
		try {
			if (collection::init(static::$arangodb->session, account::COLLECTION) && collection::init(static::$arangodb->session, market::COLLECTION)) {
				// Инициализированы коллекции

				// Search the session data in ArangoDB
				$markets = collection::search(static::$arangodb->session, sprintf(
					<<<AQL
						FOR account IN %s
							FILTER account.status != 'deleted'
							%s
							LET market = (FOR market IN OUTBOUND account._id account_edge_market FILTER market.status != 'deleted' LIMIT 1 SORT market.created DESC RETURN market)[0]
							FILTER market != null
							SORT %s
							LIMIT %d, %d
							RETURN {account, market}					
					AQL,
					account::COLLECTION,
					$before,
					$sort,
					--$page <= 0 ? 0 : $amount * $page,
					$amount
				));

				// Exit (success)
				return empty($markets) ? [] : (is_array($markets) ? $markets : [$markets]);
			} else throw new exception('Не удалось инициализировать коллекции');
		} catch (exception $e) {
			// Write to the errors registry
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Exit (fail)
		return [];
	}
}
